package main

import (
	"context"
	"fmt"
	"os"
	"os/signal"
	"syscall"

	multiaddr "gitlab.dms3.io/mf/go-multiaddr"
	p2p "gitlab.dms3.io/p2p/go-p2p"
	peerstore "gitlab.dms3.io/p2p/go-p2p-core/peer"
	"gitlab.dms3.io/p2p/go-p2p/p2p/protocol/ping"
)

func main() {
	// create a background context (i.e. one that never cancels)
	ctx := context.Background()

	// start a p2p node that listens on TCP port 2000 on the IPv4
	// loopback interface
	node, err := p2p.New(ctx,
		// p2p.ListenAddrStrings("/ip4/127.0.0.1/tcp/2000"),
		p2p.ListenAddrStrings("/ip4/127.0.0.1/tcp/0"),
		p2p.Ping(false),
	)
	if err != nil {
		panic(err)
	}

	// configure our own ping protocol
	pingService := &ping.PingService{Host: node}
	node.SetStreamHandler(ping.ID, pingService.PingHandler)

	// print the node's PeerInfo in multiaddr format
	peerInfo := peerstore.AddrInfo{
		ID:    node.ID(),
		Addrs: node.Addrs(),
	}
	addrs, err := peerstore.AddrInfoToP2pAddrs(&peerInfo)
	fmt.Println("p2p node address:", addrs[0])
	fmt.Println("p2p node ID:", node.ID())
	fmt.Println("p2p node Host:", node.Mux())

	// if a remote peer has been passed on the command line, connect to it
	// and send it 5 ping messages, otherwise wait for a signal to stop
	if len(os.Args) > 1 {
		addr, err := multiaddr.NewMultiaddr(os.Args[1])
		if err != nil {
			panic(err)
		}
		peer, err := peerstore.AddrInfoFromP2pAddr(addr)
		if err != nil {
			panic(err)
		}
		if err := node.Connect(ctx, *peer); err != nil {
			panic(err)
		}
		fmt.Println("sending 5 ping messages to", addr)
		ch := pingService.Ping(ctx, peer.ID)
		for i := 0; i < 5; i++ {
			res := <-ch
			fmt.Println("got ping response!", "RTT:", res.RTT)
		}
	} else {
		// wait for a SIGINT or SIGTERM signal
		ch := make(chan os.Signal, 1)
		signal.Notify(ch, syscall.SIGINT, syscall.SIGTERM)
		<-ch
		fmt.Println("Received signal, shutting down...")
	}

	// shut the node down
	if err := node.Close(); err != nil {
		panic(err)
	}
}
